import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.util.LinkedList;
import java.util.PriorityQueue;

/**
 * Eine Klasse zur Darstellung eines Graphen.
 * Fuer nicht gerichtete Kanten muessen beide gerichtet erzeugt werden.
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Graph
{
    private int breite, hoehe, anzahl;
    private String farbe;
    int[][] knotenDaten10={{10,10},{220,30},{60,110},{110,280},{270,190},
    {20,450},{200,300},{180,490},{400,20},{480,400}};
    int[][] kantenDaten10={{1,2},{1,3},{1,4},{1,6},{1,9},{2,3},{2,4},{2,5},{2,7},//{2,9}, // alternativ!
    {3,4},{3,7},{4,6},{4,8},{5,9},{5,10},{6,8},{7,8},{7,10},{8,9},{8,10},{9,10}};

    /**
     * Erzeuge eine neue Knotenliste mit anzahl vielen Punkten.
     * Die Kante nmuessen gesondert erzeugt werden.
     */
    public Graph(int anzahl, int breite, int hoehe)
    {
        this.anzahl = anzahl;
        this.breite = breite;
        this.hoehe = hoehe;
        Leinwand.gibLeinwand(breite, hoehe);
        farbe = "grau";
        Kante.setzeDimension(breite, hoehe);
        Knoten.setzeDimension(breite, hoehe);
        if (anzahl==10)
        {
            for (int i=0; i<10; i++) new Knoten(knotenDaten10[i][0],knotenDaten10[i][1]);
            for (int i=0; i<kantenDaten10.length; i++)
            {
                new Kante(Knoten.gibKnoten("Knoten_"+kantenDaten10[i][0]),
                          Knoten.gibKnoten("Knoten_"+kantenDaten10[i][1]));
                new Kante(Knoten.gibKnoten("Knoten_"+kantenDaten10[i][1]),
                          Knoten.gibKnoten("Knoten_"+kantenDaten10[i][0]));
                        }
        }
        else for (int i=0;i<anzahl;i++){ new Knoten(); }
    }
    
    /**
     * Erzeuge einen Testgraphen mit 5 Knoten.
     */
    public Graph()
    {
        this.anzahl = 5;
        breite = 300;
        hoehe = 300;
        //kantenwerte = new int[5][3];
        farbe = "blau";
        Kante.setzeDimension(breite, hoehe);
        Knoten.setzeDimension(breite, hoehe);
        Knoten k1 = new Knoten(10, 10);
        Knoten k2 = new Knoten(220, 30);
        Knoten k3 = new Knoten(60, 110);
        Knoten k4 = new Knoten(110, 280);
        Knoten k5 = new Knoten(270, 190);
        new Kante(k1, k2);new Kante(k2, k1);
        new Kante(k1, k3);new Kante(k3, k1);
        new Kante(k1, k4);new Kante(k4, k1);
        new Kante(k2, k3);new Kante(k3, k2);
        new Kante(k2, k4);new Kante(k4, k2);
        new Kante(k2, k5);new Kante(k5, k2);
        new Kante(k3, k4);new Kante(k4, k3);
        new Kante(k4, k5);new Kante(k5, k4);
    }
    
    /**
     * verknuepft alle Knoten zu einem Graphen.
     */
    public void vollstaendig(){
        Kante.loescheKanten();
        for (int i=0;i<anzahl;i++)
        for (int j=i+1;j<anzahl;j++)
        {
            new Kante(Knoten.gibKnoten(i),Knoten.gibKnoten(j));
            new Kante(Knoten.gibKnoten(j),Knoten.gibKnoten(i));
        }
    }
    
    // **************** Abschnitt fuer die Grafik *****************
    /**
     * Berechnet den zu zeichnenden Graphen anhand der gegebenen Daten.
     */
    protected Shape gibAktuelleFigur()
    {
        GeneralPath graph = new GeneralPath();
        for (int i=0;i<Kante.gibAnzahl();i++){
            graph.append(Kante.gibKante(i), false);
        }
        return graph;
    }
  
    /**
     * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
     */
    public void zeichne()
    {
        Shape figur = gibAktuelleFigur();
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.zeichne (
          this,           // leinwand kennt das Objekt
          farbe,          // definiert seine Zeichenfarbe
          figur);         // definiert seinen grafischen Aspekt
    }

    /**
     * Loesche diesen Graphen vom Bildschirm.
     */
    public void loesche()
    {
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.entferne(this);
    }
}

